'''### _Subsurface data input_
Contains data processing functions used in the containers.
Some of the scripts are dependent on FMU postprocessing scripts
that will be made open source in the near future.
'''

from ._history_match import extract_mismatch, scratch_ensemble
from ._intersect import load_surface, get_wfence, get_hfence
from ._summary_stats import get_summary_data, get_summary_stats
from ._inplace_volumes import extract_volumes


__all__ = ['scratch_ensemble',
           'extract_mismatch',
           'load_surface',
           'get_wfence',
           'get_hfence',
           '_summary_stats',
           'extract_volumes']



from uuid import uuid4
import pandas as pd
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output, State
import dash_daq as daq
from webviz_config.webviz_store import webvizstore
from webviz_config.common_cache import cache
from webviz_config.containers import WebvizContainer
from ..datainput import scratch_ensemble


class Widgets:

    @staticmethod
    def dropdown_from_dict(dom_id, d):
        return dcc.Dropdown(id=dom_id,
                            options=[{'label': k, 'value': v}
                                     for k, v in d.items()],
                            value=list(d.values())[0])


class ParameterDistribution(WebvizContainer):
    '''### Parameter distribution

This container shows parameter distribution as histogram,
and correlation between the parameters as a correlation matrix.

* `ensembles`: Which ensembles in `container_settings` to visualize.
'''

    def __init__(self, app, container_settings, ensembles):

        self.ensembles = {ens: container_settings['scratch_ensembles'][ens]
                          for ens in ensembles}
        self.uid = f'{uuid4()}'
        self.p1_drop_id = f'p1-dropd-{self.uid}'
        self.p2_drop_id = f'p2-dropd-{self.uid}'
        self.scatter_id = f'scatter-id-{self.uid}'
        self.scatter_color_id = f'scatter-color-id-{self.uid}'
        self.scatter_size_id = f'scatter-size-id-{self.uid}'
        self.matrix_id = f'chart-id-{self.uid}'
        self.ens_matrix_id = f'ens-matrix-id-{self.uid}'
        self.ens_p1_id = f'ens-p1-id-{self.uid}'
        self.ens_p2_id = f'ens-p2-id-{self.uid}'
        self.density_id = f'density-id-{self.uid}'

        self.set_callbacks(app)

    @property
    def p_cols(self):
        dfs = [get_parameters(ens) for ens in self.ensembles.values()]
        return sorted(list(pd.concat(dfs).columns))

    @property
    def matrix_plot(self):
        return dcc.Graph(
            id=self.matrix_id,
            clickData={'points': [
                {'x': self.p_cols[0],
                 'y': self.p_cols[0]}]},
            config={
                'displaylogo': False,
                'modeBarButtonsToRemove': ['sendDataToCloud']
            })

    @property
    def control_div(self):
        return html.Div(
            style={'padding-top': 10, 'padding-left': 100},
            children=[
                html.Div(
                    children=[
                        html.Label('Set ensemble in all plots:',
                                   style={'font-weight': 'bold'}),
                        html.Div(style={'padding-bottom': 20},
                                 children=[
                            Widgets.dropdown_from_dict(
                                self.ens_matrix_id, self.ensembles)
                        ]),
                        html.Label('Parameter horisontal axis', style={
                            'font-weight': 'bold'}),
                        html.Div(
                            style={'padding-bottom': 20, 'display': 'grid',
                                   'grid-template-columns': '3fr 2fr'},
                            children=[
                                dcc.Dropdown(
                                    id=self.p1_drop_id,
                                    options=[{'label': p, 'value': p}
                                             for p in self.p_cols],
                                    value=self.p_cols[0]),
                                Widgets.dropdown_from_dict(
                                    self.ens_p1_id, self.ensembles),
                            ]),
                        html.Label('Parameter vertical axis', style={
                            'font-weight': 'bold'}),
                        html.Div(style={
                            'padding-bottom': 20,
                            'display': 'grid',
                            'grid-template-columns': '3fr 2fr'},
                            children=[
                            dcc.Dropdown(
                                id=self.p2_drop_id,
                                options=[{'label': p, 'value': p}
                                         for p in self.p_cols],
                                value=self.p_cols[0]),
                            Widgets.dropdown_from_dict(
                                self.ens_p2_id, self.ensembles)
                        ]),
                        html.Label('Color scatter by',
                                   style={'font-weight': 'bold'}),
                        dcc.Dropdown(
                            id=self.scatter_color_id,
                            options=[{'label': p, 'value': p}
                                     for p in self.p_cols])]),
                html.Div(style={'padding-top': 20, 'display': 'grid',
                                'grid-template-columns': '3fr 1fr 4fr'},
                         children=[
                    html.Label('Show density plot',
                               style={'font-weight': 'bold'}),
                    daq.ToggleSwitch(id=self.density_id, value=True)])
            ])

    @property
    def layout(self):
        return html.Div(
            children=[
                html.Div(
                    style={'display': 'grid',
                           'grid-template-columns': '3fr 2fr'},
                    children=[
                        html.Div(
                            children=[self.matrix_plot]), self.control_div]),
                dcc.Graph(
                    id=self.scatter_id,
                    config={
                        'displaylogo': False,
                        'modeBarButtonsToRemove': ['sendDataToCloud']
                    }),
            ])

    def set_callbacks(self, app):
        @app.callback(Output(self.matrix_id, 'figure'),
                      [Input(self.ens_matrix_id, 'value'),
                       Input(self.p1_drop_id, 'value'),
                       Input(self.p2_drop_id, 'value')])
        def update_matrix(ens, p1, p2):
            '''Renders correlation matrix.
            Currently also re-renders matrix to update currently
            selected cell. This is not optimal, but hard to prevent
            as an Output object only can have one callback attached,
            and it is not possible to assign callbacks to individual
            elements of a Plotly graph object
            '''
            fig = render_matrix(ens)
            # Finds index of the currently selected cell
            x_index = list(fig['data'][0]['x']).index(p1)
            y_index = list(fig['data'][0]['y']).index(p2)
            # Adds a shape to highlight the selected cell
            shape = [
                {
                    'xref': 'x',
                    'yref': 'y',
                    'x0': x_index-0.5,
                    'y0': y_index-0.5,
                    'x1': x_index+0.5,
                    'y1': y_index+0.5,
                    'type': 'rect',
                    'line': {
                        'color': 'black',
                    }
                }]
            fig['layout']['shapes'] = shape
            return fig

        @app.callback(Output(self.scatter_id, 'figure'),
                      [Input(self.ens_p1_id, 'value'),
                       Input(self.p1_drop_id, 'value'),
                       Input(self.ens_p2_id, 'value'),
                       Input(self.p2_drop_id, 'value'),
                       Input(self.scatter_color_id, 'value'),
                       Input(self.density_id, 'value')])
        def update_scatter(ens1, p1, ens2, p2, color, density):
            return render_scatter(ens1, p1, ens2, p2, color, density)

        @app.callback([Output(self.p1_drop_id, 'value'),
                       Output(self.p2_drop_id, 'value'),
                       Output(self.ens_p1_id, 'value'),
                       Output(self.ens_p2_id, 'value')],
                      [Input(self.matrix_id, 'clickData'),
                       Input(self.ens_matrix_id, 'value')]
                      )
        def update_from_click(cd, ens):
            try:
                points = cd['points'][0]
            # TypeError is returned if no cells are clicked
            except TypeError:
                return [None for i in range(4)]

            return [points['x'], points['y'], ens, ens]

    def add_webvizstore(self):
        return [([get_parameters, [{'ensemble_path': v}
                                   for v in self.ensembles.values()]])]


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def get_parameters(ensemble_path) -> pd.DataFrame:

    return scratch_ensemble('', ensemble_path).parameters


@cache.memoize(timeout=cache.TIMEOUT)
def render_scatter(ens1, x_col, ens2, y_col, color, density):
    if ens1 == ens2:
        real_text = [f'Realization:{r}'
                     for r in get_parameters(ens1)['REAL']]
    else:
        real_text = [f'Realization:{r}(x)'
                     for r in get_parameters(ens2)['REAL']]

    x = get_parameters(ens1)[x_col]
    y = get_parameters(ens2)[y_col]
    color = get_parameters(ens1)[color] if color else None
    data = []
    data.append({
        'x': x,
        'y': y,
        'marker': {
            'color': color
        },
        'text': real_text,
        'type': 'scatter',
        'mode': 'markers',
        'showlegend': False
    })
    data.append({
        'x': x,
        'type': 'histogram',
        'yaxis': 'y2',
        'showlegend': False,
        'marker': {'color': 'rgb(31, 119, 180)'},
    })
    data.append({
        'y': y,
        'type': 'histogram',
        'xaxis': 'x2',
        'showlegend': False,
        'marker': {'color': 'rgb(31, 119, 180)'}
    })
    if density:
        data.append({
            'x': x,
            'y': y,
            'hoverinfo': 'none',
            'autocolorscale': False,
            'showlegend': False,
            'colorscale': [
                [0, 'rgb(255,255,255)'],
                [0.125, 'rgb(37, 52, 148)'],
                [0.25, 'rgb(34, 94, 168)'],
                [0.375, 'rgb(29, 145, 192)'],
                [0.5, 'rgb(65, 182, 196)'],
                [0.625, 'rgb(127, 205, 187)'],
                [0.75, 'rgb(199, 233, 180)'],
                [0.875, 'rgb(237, 248, 217)'],
                [1, 'rgb(255, 255, 217)']],
            'contours': {
                'coloring': 'fill',
                # 'end': 80.05,
                'showlines': True,
                'size': 5,
                'start': 5
            },
            'name': 'density',
            'ncontours': 20,
            'reversescale': False,
            'showscale': False,
            'type': 'histogram2dcontour',
        })
    layout = {
        'margin': {'t': 20, 'b': 50, 'l': 200, 'r': 200},
        'bargap': 0.05,
        'xaxis': {
            'title': x_col,
            'domain': [0, 0.85],
            'showgrid': False,
            'showline': False,
            'zeroline': False,
            'showlegend': False
        },
        'xaxis2': {
            'domain': [0.85, 1],
            'showgrid': False,
            'showline': False,
            'zeroline': False,
            'showticklabels': False
        },
        'yaxis': {
            'title': y_col,
            'domain': [0, 0.85],
            'showgrid': False,
            'zeroline': False,

        },
        'yaxis2': {
            'domain': [0.85, 1],
            'showgrid': False,
            'zeroline': False,
            'showticklabels': False,
            'showline': False
        }
    }

    return {'data': data, 'layout': layout}


@cache.memoize(timeout=cache.TIMEOUT)
def render_matrix(ensemble_path):

    data = get_parameters(ensemble_path)
    values = list(data.corr().values)

    data = {
        'type': 'heatmap',
        'x': data.columns,
        'y': data.columns,
        'z': values,
        'zmin': -1,
        'zmax': 1
    }

    layout = {
        'uirevision': 'keep_matrix',
        'title': 'Pairwise correlation matrix',
        'margin': {'t': 50, 'b': 50},
        'xaxis': {'ticks': '', 'showticklabels': False},
        'yaxis': {'ticks': '', 'showticklabels': False},
    }

    return {'data': [data], 'layout': layout}



import os
from uuid import uuid4
import pandas as pd
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output
from webviz_config.webviz_store import webvizstore
from webviz_config.common_cache import cache
from webviz_config.containers import WebvizContainer


class DiskUsage(WebvizContainer):
    '''### Disk usage

This container adds functionality for standard visualization of disk usage in
FMU projects. It adds a dashboard element where the user can choose between
showing disk usage, per user, either as a pie chart or as a bar chart.

* `scratch_dir`: Path to the directory you want to show disk usage for, e.g.
  `/scratch/fmu`.
* `title`: Optional title for the container.
'''

    def __init__(self, app, scratch_dir: str):

        self.scratch_dir = scratch_dir
        self.chart_id = 'chart-id-{}'.format(uuid4())
        self.plot_type_id = 'plot-type-id-{}'.format(uuid4())
        self.disk_usage = get_disk_usage(self.scratch_dir)
        self.date = str(self.disk_usage['date'].unique()[0])
        self.users = self.disk_usage['userid']
        self.usage = self.disk_usage['usageKB']/(1024**2)
        self.set_callbacks(app)

    @property
    def layout(self):
        return html.Div([
                    html.P(
                        f'This is the disk usage on \
                        {self.scratch_dir} per user, \
                        as of {self.date}.'),
                    dcc.RadioItems(
                        id=self.plot_type_id,
                        options=[
                            {'label': i, 'value': i}
                            for i in ['Pie chart', 'Bar chart']],
                        value='Pie chart'),
                    dcc.Graph(
                        id=self.chart_id,
                        config={
                                'displaylogo': False,
                                'modeBarButtonsToRemove': ['sendDataToCloud']
                               }
                             )
               ])

    def set_callbacks(self, app):
        @app.callback(Output(self.chart_id, 'figure'),
                      [Input(self.plot_type_id, 'value')])
        def update_plot(plot_type):
            if plot_type == 'Pie chart':
                data = [{
                    'values': self.usage,
                    'labels': self.users,
                    'text': (self.usage).map('{:.2f} GB'.format),
                    'textinfo': 'label',
                    'textposition': 'inside',
                    'hoverinfo': 'label+text',
                    'type': 'pie'
                }]
                layout = {}

            elif plot_type == 'Bar chart':
                data = [{
                    'y': self.usage,
                    'x': self.users,
                    'text': (self.usage).map('{:.2f} GB'.format),
                    'hoverinfo': 'x+text',
                    'type': 'bar'
                }]
                layout = {
                    'yaxis': {
                        'title': 'Usage in Gigabytes',
                        'family': 'Equinor'
                    },
                    'xaxis': {
                        'title': 'User name',
                        'family': 'Equinor'
                    },
                }

            layout['height'] = 800
            layout['width'] = 1000
            layout['font'] = {'family': 'Equinor'}
            layout['hoverlabel'] = {'font': {'family': 'Equinor'}}

            return {'data': data, 'layout': layout}

    def add_webvizstore(self):
        return [(get_disk_usage, [{'scratch_dir': self.scratch_dir}])]


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def get_disk_usage(scratch_dir) -> pd.DataFrame:
    try:
        df = pd.read_csv(os.path.join(scratch_dir, 'disk_usage.csv'))
    except FileNotFoundError:
        raise FileNotFoundError(f'No disk usage file found at {scratch_dir}')

    last_date = sorted(list(df['date'].unique()))[-1]
    return df.loc[df['date'] == last_date]



import os
try:
    import fmu.ensemble
except ImportError:  # fmu.ensemble is an optional dependency, e.g.
    pass             # for a portable webviz instance, it is never used.

import pandas as pd
from webviz_config.common_cache import cache
from webviz_config.webviz_store import webvizstore


@cache.memoize(timeout=cache.TIMEOUT)
def scratch_ensemble(ensemble_name, ensemble_path):
    return fmu.ensemble.ScratchEnsemble(ensemble_name, ensemble_path)


@cache.memoize(timeout=cache.TIMEOUT)
@webvizstore
def extract_volumes(ensemble_paths, volfolder, volfiles) -> pd.DataFrame:
    dfs = []
    for ens_name, ens_path in list(ensemble_paths):
        ens_dfs = []
        ens = scratch_ensemble(ens_name, ens_path)
        for volname, volfile in volfiles:
            path = os.path.join(volfolder, volfile)
            df = ens.load_csv(path)
            df['SOURCE'] = volname
            df['ENSEMBLE'] = ens_name
            ens_dfs.append(df)
        dfs.append(pd.concat(ens_dfs))
    return pd.concat(dfs)



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

tests_require = [
    'chromedriver-binary>=74.0.3729.6.0',
    'ipdb',
    'percy',
    'selenium>=3.141.0',
    'flake8',
    'pylint',
    'mock'
]

setup(
    name='webviz-subsurface',
    description='Webviz config containers for subsurface data',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/equinor/webviz-subsurface',
    author='R&T Equinor',
    packages=find_packages(exclude=['tests']),
    entry_points={
        'webviz_config_containers': [
            'SummaryStats = webviz_subsurface.containers:SummaryStats',
            'ParameterDistribution = webviz_subsurface.containers:ParameterDistribution',
            'DiskUsage = webviz_subsurface.containers:DiskUsage',
            'SubsurfaceMap = webviz_subsurface.containers:SubsurfaceMap',
            'HistoryMatch = webviz_subsurface.containers:HistoryMatch',
            'Intersect = webviz_subsurface.containers:Intersect',
            'MorrisPlot = webviz_subsurface.containers:MorrisPlot',
            'InplaceVolumes = webviz_subsurface.containers:InplaceVolumes'
        ]
    },
    install_requires=[
        'scipy>=1.2.1',
        'dash-daq>=0.1.5',
        'webviz-config>=0.0.4',
        'webviz-plotly>=0.0.1',
        'webviz-subsurface-components>=0.0.3'
    ],
    tests_require=tests_require,
    extras_require={'tests': tests_require},
    setup_requires=['setuptools_scm>=3.2.0'],
    use_scm_version=True,
    zip_safe=False
)



import mock
import dash
import pandas as pd
from webviz_config.common_cache import cache
from webviz_config.containers import InplaceVolumes

# mocked functions
extract_volumes = 'webviz_subsurface.containers'\
    '._inplace_volumes.extract_volumes'


def test_inplace_volumes(dash_duo):

    app = dash.Dash(__name__)
    app.css.config.serve_locally = True
    app.scripts.config.serve_locally = True
    app.config.suppress_callback_exceptions = True
    cache.init_app(app.server)
    container_settings = {'scratch_ensembles': {'iter-0': '', 'iter-1': ''}}
    ensembles = ['iter-0', 'iter-1']
    volfiles = {'geogrid': 'geogrid--oil.csv', 'simgrid': 'simgrid--oil.csv'}

    with mock.patch(extract_volumes) as mock_volumes:
        mock_volumes.return_value = pd.read_csv('tests/data/volumes.csv')

        vol = InplaceVolumes(app, container_settings, ensembles, volfiles)

        app.layout = vol.layout
        dash_duo.start_server(app)

        if 'Stock Tank Oil Initially Inplace' not in dash_duo.wait_for_element(
                f'#{vol.response_id}').text:
            raise AssertionError()

        if dash_duo.get_logs() != []:
            raise AssertionError()



import itertools
from uuid import uuid4
import dash_html_components as html
import dash_core_components as dcc
from dash.dependencies import Input, Output
from webviz_plotly.graph_objs import FanChart
from webviz_config.common_cache import cache
from webviz_config.containers import WebvizContainer
from plotly.colors import DEFAULT_PLOTLY_COLORS
from ..datainput import get_summary_data, get_summary_stats


class SummaryStats(WebvizContainer):

    """Summary statistics

    Args:
        ensembles: list of ensemble paths. Length >= 1
        column_keys: list of pre defined vectors to visualize. Default is none
        sampling: Optional. Either 'monthly' or 'yearly'. Default is 'monthly'.
        title: Optional title for the container.
        history_uncertainty: boolean value if history vector is subjected to
            uncertainty.
    Return:
        dcc.Graph() of either statistics plot or realization plot.
    Annotations:
        Function argument-litst are passed as tuple as they are hashed in
        webviz-store --portable option (list-type is not hashable).
        Pandas within fmu.ensemlbe expects list. Therefore argument-tuples are
        converted back into lists within get_smry_data and get_smry_stats
        functions.
    """

    def __init__(
            self,
            app,
            container_settings,
            ensembles,
            column_keys=None,
            sampling: str = 'monthly',
            history_uncertainty: bool = False):

        self.uid = f'{uuid4()}'
        self.dropwdown_vector_id = f'dropdown-vector-{self.uid}'
        self.column_keys = tuple(column_keys) if isinstance(
            column_keys, (list, tuple)) else None
        self.sampling = sampling
        self.radio_plot_type_id = f'radio-plot-type-{self.uid}'
        self.show_history_uncertainty_id = \
            f'show-history-uncertainty-{self.uid}'
        self.chart_id = f'chart-id-{self.uid}'
        self.ensemble_paths = tuple(
            (ensemble,
             container_settings['scratch_ensembles'][ensemble])
            for ensemble in ensembles)
        self.history_uncertainty = history_uncertainty
        self.vector_columns = sorted(
            list(
                get_summary_data(
                    ensemble_paths=self.ensemble_paths,
                    sampling=self.sampling,
                    column_keys=self.column_keys) .drop(
                        columns=[
                            'DATE',
                            'REAL',
                            'ENSEMBLE']).columns))
        self.smry_history_columns = tuple(
            [vctr + 'H' for vctr in self.vector_columns
             if vctr + 'H' in self.vector_columns])
        self.smry_vector_columns = tuple(
            [vctr for vctr in self.vector_columns
             if vctr not in self.smry_history_columns])
        self.set_callbacks(app)

    @property
    def layout(self):
        return html.Div([
            html.P('Summary Vector:', style={'font-weight': 'bold'}),
            dcc.Dropdown(id=self.dropwdown_vector_id,
                         clearable=False,
                         options=[{'label': i, 'value': i}
                                  for i in self.smry_vector_columns],
                         value=self.smry_vector_columns[0]),
            html.P('Plot type:', style={'font-weight': 'bold'}),
            dcc.RadioItems(id=self.radio_plot_type_id,
                           options=[{'label': i, 'value': i}
                                    for i in ['Realizations', 'Statistics']],
                           value='Realizations'),
            dcc.Checklist(
                id=self.show_history_uncertainty_id,
                options=[{'label': 'Show history', 'value': 'SHOW_H'}],
                values=[],
            ),
            html.Div(id=self.chart_id)
        ])

    def set_callbacks(self, app):
        @app.callback(Output(self.chart_id, 'children'),
                      [Input(self.dropwdown_vector_id, 'value'),
                       Input(self.radio_plot_type_id, 'value'),
                       Input(self.show_history_uncertainty_id, 'values')])
        def update_plot(
                vector,
                summary_plot_type,
                show_history_uncertainty_id):
            if summary_plot_type == 'Realizations':
                return render_realization_plot(
                    ensemble_paths=self.ensemble_paths,
                    column_keys=self.column_keys,
                    sampling=self.sampling,
                    smry_history_columns=self.smry_history_columns,
                    history_uncertainty=self.history_uncertainty,
                    vector=vector,
                    show_history_uncertainty=show_history_uncertainty_id)
            if summary_plot_type == 'Statistics':
                return render_stat_plot(
                    ensemble_paths=self.ensemble_paths,
                    column_keys=self.column_keys,
                    sampling=self.sampling,
                    vector=vector)

    def add_webvizstore(self):
        return [(get_summary_data,
                 [{'ensemble_paths': self.ensemble_paths,
                   'column_keys': self.column_keys,
                   'sampling': self.sampling, }]),
                (get_summary_stats,
                 [{'ensemble_paths': self.ensemble_paths,
                   'column_keys': self.column_keys,
                   'sampling': self.sampling}])]


def trace_group(ens_smry_data, ens, vector, color):

    ens_traces = []

    # 1st and only trace of the legendgroup to show up in legend
    ens_traces.append({
        'x': ens_smry_data[ens_smry_data['REAL']
                           == ens_smry_data.REAL.unique()[0]]['DATE'],
        'y': ens_smry_data[ens_smry_data['REAL']
                           == ens_smry_data.REAL.unique()[0]][vector],
        'legendgroup': ens,
        'name': ens,
        'type': 'markers',
        'marker': {
            'color': color
        },
        'showlegend': True
    })

    for real in ens_smry_data.REAL.unique()[1:]:

        ens_traces.append({
            'x': ens_smry_data[ens_smry_data['REAL'] == real]['DATE'],
            'y': ens_smry_data[ens_smry_data['REAL'] == real][vector],
            'legendgroup': ens,
            'name': ens,
            'type': 'line',
            'marker': {
                'color': color
            },
            'showlegend': False
        })

    return ens_traces


def single_trace(ens_smry_data, ens, vector, color):

    return {
        'x': ens_smry_data[ens_smry_data['REAL']
                           == ens_smry_data.REAL.unique()[0]]['DATE'],
        'y': ens_smry_data[ens_smry_data['REAL']
                           == ens_smry_data.REAL.unique()[0]][vector],
        'legendgroup': ens,
        'name': ens,
        'type': 'markers',
        'marker': {
            'color': color
        },
        'showlegend': True
    }


@cache.memoize(timeout=cache.TIMEOUT)
def render_realization_plot(ensemble_paths: tuple, sampling: str,
                            column_keys: tuple, vector: str,
                            smry_history_columns: tuple,
                            history_uncertainty: bool,
                            show_history_uncertainty: str):
    """ Creates scatter-plot-traces for choosen vector. One trace per
    realization will be created. If history-data are not subjeted to
    uncertainty only one traces per ensemble will be created for history_data.
    If there is no history-vector in for the given vector, no trace will be
    created. """

    cycle_list = itertools.cycle(DEFAULT_PLOTLY_COLORS)
    history_vector = (vector + 'H')

    if history_vector in smry_history_columns:
        smry_data = get_summary_data(
            ensemble_paths=ensemble_paths,
            column_keys=column_keys,
            sampling=sampling)[
                ['REAL', 'DATE', 'ENSEMBLE', vector, history_vector]]
    else:
        smry_data = get_summary_data(
            ensemble_paths=ensemble_paths,
            column_keys=column_keys,
            sampling=sampling)[
                ['REAL', 'DATE', 'ENSEMBLE', vector]]

    smry_data.dropna(subset=[vector])

    plot_traces = []

    for ens in smry_data.ENSEMBLE.unique():

        plot_traces += trace_group(
            ens_smry_data=smry_data[smry_data['ENSEMBLE'] == ens],
            ens=ens,
            vector=vector,
            color=next(cycle_list))

        if (history_vector in smry_history_columns
                and 'SHOW_H' in show_history_uncertainty
                and history_uncertainty):

            plot_traces += trace_group(
                ens_smry_data=smry_data[smry_data['ENSEMBLE'] == ens],
                ens=ens,
                vector=history_vector,
                color='black')

        if (history_vector in smry_history_columns
                and 'SHOW_H' in show_history_uncertainty
                and not history_uncertainty):

            plot_traces += single_trace(
                ens_smry_data=smry_data[smry_data['ENSEMBLE'] == ens],
                ens=ens,
                vector=history_vector,
                color='black')

    layout = {
        'hovermode': 'closest',
        'barmode': 'overlay',
        'bargap': 0.05,
        'xaxis': {'title': 'Date', 'family': 'Equinor'},
        'yaxis': {'title': vector, 'family': 'Equinor'},
        'font': {'family': 'Equinor'},
        'hoverlabel': {'font': {'family': 'Equinor'}},
    }

    return dcc.Graph(figure={'data': plot_traces, 'layout': layout},
                     config={
                         'displaylogo': False,
                         'modeBarButtonsToRemove': ['sendDataToCloud']})


@cache.memoize(timeout=cache.TIMEOUT)
def render_stat_plot(ensemble_paths: tuple, sampling: str, column_keys: tuple,
                     vector: str):
    """Loops over ensemlbe_paths and creates one summary_statistics-plot per
    ensemble."""

    smry_stats = get_summary_stats(
        ensemble_paths=ensemble_paths,
        column_keys=column_keys,
        sampling=sampling)

    fan_chart_divs = []
    for ens in smry_stats.ENSEMBLE.unique():
        vector_stats = smry_stats[
            smry_stats['ENSEMBLE'] == ens][
                vector].unstack().transpose()
        vector_stats['name'] = vector
        vector_stats.rename(index=str, inplace=True,
                            columns={"minimum": "min", "maximum": "max"})
        fan_chart_divs.append(html.H5(ens))
        fan_chart_divs.append(
            html.Div(
                dcc.Graph(
                    id='graph-{}'.format(ens),
                    figure=FanChart(vector_stats.iterrows()),
                    config={
                        'displaylogo': False,
                        'modeBarButtonsToRemove': ['sendDataToCloud']
                    }
                )
            )
        )

    return fan_chart_divs
